from dataclasses import dataclass
from decimal import Decimal
import decimal

from logging import getLogger

logger = getLogger(__name__)

from typing import NamedTuple

class GridPrices(NamedTuple):
    initial: Decimal
    grid: list[Decimal]

def compute_grid_prices( * ,
    price_decimal_places: int,
    center_price: Decimal,
    gap: Decimal,
    order_count: int,
) -> GridPrices:
    with decimal.localcontext() as context:
        context.rounding = decimal.ROUND_DOWN
        initial_price = round(center_price, price_decimal_places)
    return compute_grid_prices_skewed(
        price_decimal_places = price_decimal_places,
        center_price = center_price,
        low_price = initial_price - (Decimal(order_count / 2)) * gap,
        gap = gap,
        order_count = order_count,
    )


def compute_grid_prices_skewed(* ,
    price_decimal_places: int,
    center_price: Decimal,
    low_price: Decimal,
    gap: Decimal,
    order_count: int,
) -> GridPrices:
    with decimal.localcontext() as context:
        context.rounding = decimal.ROUND_DOWN
        initial_price = round(center_price, price_decimal_places)
    buy_prices = []
    for i in range(order_count):
        price = initial_price - gap * (i+1)
        if price < low_price:
            break
        buy_prices.append(price)
    sell_prices_count = order_count - len(buy_prices)
    prices = []
    for i in range(sell_prices_count):
        if buy_prices:
            prices.append(buy_prices.pop(0))
        prices.append(
            initial_price + gap * (i+1)
        )
    if buy_prices:
        prices += buy_prices
    return GridPrices(initial_price, prices)


__all__ = [
    "compute_grid_prices",
    "compute_grid_prices_skewed",
]


import logging
import logging.config
from os import getenv
import pathlib
from .config import get_path

config_file = get_path("logging.conf")
if pathlib.Path(config_file).is_file():
    logging.config.fileConfig(config_file, disable_existing_loggers=False)
else:
    logging.basicConfig(
        level=getenv("LOG_LEVEL", "INFO"),
        format="%(threadName)s %(message)s",
        datefmt="[%X]",
        handlers=[logging.StreamHandler()],
    )

logger = logging



from decimal import Decimal
from ..models.orderbook import Orderbook, OrderbookEntry


def price_volume(x: OrderbookEntry):
    return Decimal(x[0]), Decimal(x[1])


def get_price_from_tuple(x: OrderbookEntry) -> Decimal:
    return Decimal(x[0])


def get_volume_from_tuple(x: OrderbookEntry) -> Decimal:
    return Decimal(x[1])


def get_bids(x: Orderbook) -> list[OrderbookEntry]:
    return x["bids"]


def get_asks(x: Orderbook) -> list[OrderbookEntry]:
    return x["asks"]


def top_bid_price(x: Orderbook) -> Decimal:
    return get_price_from_tuple(get_bids(x)[0])


def lowest_ask_price(x: Orderbook) -> Decimal:
    return get_price_from_tuple(get_asks(x)[0])



from typing import TypedDict, NamedTuple


class OrderbookEntry(NamedTuple):
    price: float
    volume: float


class Orderbook(TypedDict):
    asks: list[OrderbookEntry]
    bids: list[OrderbookEntry]
    symbol: str
    timestamp: int
    datetime: str
    nonce: str
